
// inspired by stroustrup slides
// use -lpthread

#include <vector>
#include <thread>
#include <future>
#include <iostream>

double f(const std::vector<double>& v) {
    double sum = 0;
    for(double x: v) sum+=x;
    return sum;
}

double g(const std::vector<double>& v) {
    double p = 1;
    for(double x: v) p*=x;
    return p;
}

void user(const std::vector<double>& some_vec) // note: const
{
    double res1, res2;
    std::thread t1 {[&]{ res1 = f(some_vec); }}; // lambda: leave result in res1
    std::thread t2 {[&]{ res2 = g(some_vec); }};
    std::cout << "Computation 1 in progress...\n";
    t1.join();
    t2.join();
    std::cout << res1 << ' ' << res2 << '\n';
}

// more elegant solution
void user2(const std::vector<double>& some_vec)
{
    auto res1 = std::async(f,some_vec); // run f in extra thread
    auto res2 = std::async(g,some_vec);
    std::cout << "Computation 2 in progress...\n";
    std::cout << res1.get() << ' ' << res2.get() << '\n';
}

int main() {
    // TODO: create BIG random vector and measure time
    std::vector<double> v{ 1.0, 3.14, 10.0, 50.0 };
    user(v);
    user2(v);
}
