
// initialization examples for C++11

#include <iostream>
#include <complex>
#include <vector>
#include <map>
#include <string>
#include <initializer_list>     // has priority over other constructors

using namespace std;

struct S2 {
   int m1;
   double m2, m3;
};
S2 s21 = { 1, 2, 3.0 }; // OK
#if 0
S2 s22 { 1.0, 2, 3 };   // Error: narrowing    int(1.0)
#else
S2 s22 { 1, 2, 3 };     // OK
#endif
S2 s23 { };             // OK: default to 0,0,0

void f(vector<const char*> v) {
    cout << v[0] << endl;
}
vector<const char*> g() {
    return { "alpha" };             // return list of one element
}

// TODO:
#define PRINT(x) do { cout << '\n' << #x << endl; x; }while(0)
#define PRINTARRAY(p) do { for(int i=0; i<4; i++) cout << " [" << i << "]=" << p[i]; cout << endl; }while(0)

void ff() {
#   if 1
    int a;      // O.K.
#   else
    int a();    // Error: function declaration
#   endif
    a = int();  // value-initialization
    cout << "  a=" << a << endl;
    a = int{};  // value-initialization
    cout << "  a=" << a << endl;

    int *p;

    PRINT(p = new int[4]);    // default-init: undef
    PRINTARRAY(p);
    delete[] p;

    PRINT(p = new int[4]());  // value-init: 0,...
    PRINTARRAY(p);
    delete[] p;

    PRINT(p = new int[4]{});
    PRINTARRAY(p);
    delete[] p;

    PRINT(( p = new int[4]{ 1, 2, 3, } ));
    PRINTARRAY(p);
    delete[] p;

    //new int[5](5);

    PRINT(p = new int[4]{5});
    PRINTARRAY(p);
    delete[] p;

    PRINT(p = new int(333));
    cout << "*p=" << *p << endl;
    delete p;

    PRINT(p = new int);    // default for int: uninitialized
    cout << "undef *p=" << *p << endl;
    delete p;
    PRINT(p = new int());  // value init: 0
    cout << "*p=" << *p << endl;
    delete p;
    PRINT(p = new int{5});
    cout << "*p=" << *p << endl;
    delete p;
}

int main() {
    // draft 8.5.4
    int a {};
    cout << "a=" << a << endl;

    complex<double> z{1,2};
    cout << "z=" << z << endl;

    vector<string> *sp = new vector<string>{"once", "upon", "a", "time"}; // 4 string elements
    cout << "*sp =";
    for(auto x: *sp)
        cout << " " << x;
    cout << endl;

    f( {"aaa","bbb"} );      // pass list of two elements

    int* p {};                          // initialization to zero / null pointer
    cout << "p=" << p << endl;

    double x = double{1};               // explicitly construct a double
    cout << "x=" << x << endl;

    map<string,int> anim { {"a",1}, {"c",3}, {"p",15} };
    cout << anim["c"] << endl;

    ff();
}

